#!/usr/bin/python3 -O
#############################################################################
# File      : compile.py
# Package   : rpmlint
# Author    : Frederic Lepied
# Created on    : Sat Oct 23 23:40:21 1999
# Version   : $Id: compile.py 250876 2009-01-06 21:31:46Z eugeni $
# Purpose   : byte compile all python files given in arguments.
#############################################################################

import py_compile
import sys

for f in sys.argv[2:]:
    try:
        py_compile.compile(f, f+"o", sys.argv[1] + f)
    except:
        print("Cannot compile", f)
        pass
    
# compile.py ends here
